use Irssi 20020324;
use strict;
use vars qw($VERSION %IRSSI);

$VERSION = '0.2.1';
%IRSSI = (
          authors     => 'Clemens Heidinger',
          contact     => 'spoooky@quizparanoia.org',
          name        => 'len',
          description => 'If you try to get a nick with 11 characters but only 9 are allowed, this script will prevent the nickchange. The same for too long topics, kickmsgs, partmsgs and quitmsgs.',
          license     => 'BSD',
          url         => 'http://scripts.quizparanoia.org/',
          modules     => '',
          changed     => 'Wed Oct 02 19:05:07 CEST 2002'
         );

################################################################################
#                                                                              #
# CHANGELOG                                                                    #
#                                                                              #
# 2002-09-26    release 0.1                                                    #
#               initial release                                                #
#                                                                              #
# 2002-09-27    release 0.2                                                    #
#               Added command /len with a table containing the values for      #
#               NICKLEN etc. and tips if these values haven't been received    #
#               from the server yet                                            #
#                                                                              #
# 2002-10-02    release 0.2.1                                                  #
#               Changed output format of /len                                  #
#                                                                              #
################################################################################

# Register commands

Irssi::command_bind('len', \&command_len);

# Register signals

Irssi::signal_add_last('event 005', \&signal_event_005);
Irssi::signal_add_first('command nick', \&signal_command_nick);
Irssi::signal_add_first('command topic', \&signal_command_topic);
Irssi::signal_add_first('command kick', \&signal_command_kick);
Irssi::signal_add_first('command part', \&signal_command_part);
Irssi::signal_add_first('command quit', \&signal_command_quit);

# Register themes

Irssi::theme_register(['len_print', '[$0] {line_start} $1']);

# Variables

# Most IRC-Server send a message containing the values for NICKLEN, TOPICLEN
# and KICKLEN.
# Well, some server do not send this message. Add them below.

my %data = (
            'IRCNet' => {
                         'nicklen'  => 9,
                         'topiclen' => 80,
                         'kicklen'  => 80,
                         'partlen'  => 80,
                         'quitlen'  => 80,
                        },
           );

# Subroutines (commands)

sub command_len {
	my ($data, $server, $witem) = @_;
	my $output;

	unless ($server and defined($server)) {
		print CLIENTCRAP "First connect to a server...";
		return;
	}

	my $nicklen  = sprintf "%-8s", $data{"$server->{'tag'}"}{'nicklen'};
	my $topiclen = sprintf "%-9s", $data{"$server->{'tag'}"}{'topiclen'};
	my $kicklen  = sprintf "%-8s", $data{"$server->{'tag'}"}{'kicklen'};
	my $partlen  = sprintf "%-8s", $data{"$server->{'tag'}"}{'partlen'};
	my $quitlen  = sprintf "%-8s", $data{"$server->{'tag'}"}{'quitlen'};

	$output = &fix(<<"	END");
	|=========|=================|
	| Thing   | Characters max. |
	|=========|=================|
	| Nick    | $nicklen        |
	|---------|-----------------|
	| Topic   | $topiclen       |
	|---------|-----------------|
	| kickmsg | $kicklen        |
	|---------|-----------------|
	| partmsg | $partlen        |
	|---------|-----------------|
	| quitmsg | $quitlen        |
	|---------|-----------------|
	END

	unless ($nicklen  =~ /\d/ &&
	        $topiclen =~ /\d/ &&
	        $kicklen  =~ /\d/ &&
	        $partlen  =~ /\d/ &&
	        $quitlen  =~ /\d/)
	{
		$output .= &fix(<<"		END");

		Obviously there are some values missing.
		When you connect to a server most send you a string containing
		the values for NICKLEN etc..
		If you loaded this script after connecting to $server->{'tag'}
		you should reconnect.
		If this didn't help, the server is not sending this message 
		with these values.
		You could try another server then.
		If you don't want that, you could find out the values and adjust
		\%data hash in the source code. \%data might look like this:

		my \%data = (
		             'IRCNet' => {
		                          'nicklen'  => 9,
		                          'topiclen' => 80,
		                          'kicklen'  => 80,
		                          'partlen'  => 80,
		                          'quitlen'  => 80,
		                         },
		             '$server->{'tag'}' => {
		                          'nicklen'  => 999,
		                          'topiclen' => 999,
		                          'kicklen'  => 999,
		                          'partlen'  => 999,
		                          'quitlen'  => 999,
		                         },
		            );
		END
	}

	foreach my $line (split /\n/, $output) {
		Irssi::printformat(MSGLEVEL_CLIENTCRAP, 'len_print', $server->{'tag'}, $line);
	}
}

# Subroutines (signals)

# Most server send this message containig the values for NICKLEN etc. on
# connect (event 005).

sub signal_event_005 {
	my ($server, $string) = @_;

	my ($nicklen)  = $string =~ /NICKLEN=(\d+)/;
	my ($topiclen) = $string =~ /TOPICLEN=(\d+)/;
	my ($kicklen)  = $string =~ /KICKLEN=(\d+)/;
	my $partlen = $kicklen;
	my $quitlen = $kicklen;

	$data{"$server->{'tag'}"} = {
	                             'nicklen'  => $nicklen,
	                             'topiclen' => $topiclen,
	                             'kicklen'  => $kicklen,
	                             'partlen'  => $partlen,
	                             'quitlen'  => $quitlen,
	                            };
}

sub signal_command_nick {
	my ($nick, $server, $witem) = @_;

	return unless ($server and defined($server));

	my $len = length($nick);
	my $maxlen = $data{"$server->{'tag'}"}{'nicklen'};

	if ($maxlen > 0 && $len > $maxlen) {
		print CLIENTCRAP "len.pl: Nick too long! ($len/$maxlen)";
		Irssi::signal_stop();
	}
}

sub signal_command_topic {
	my ($command, $server, $witem) = @_;

	return unless ($server and defined($server));

	# Syntax for /topic:
	# TOPIC [-delete] [<channel>] [<topic>]
	# We want to isolate <reason> to know how long it is

	# Delete <channel>
	$command =~ s/^#\S+ //;

	# The rest of $command is the topic
	my $topic = $command;

	my $len = length($topic);
	my $maxlen = $data{"$server->{'tag'}"}{'topiclen'};

	if ($maxlen > 0 && $len > $maxlen) {
		print CLIENTCRAP "len.pl: Topic too long! ($len/$maxlen)";
		Irssi::signal_stop();
	}
}

sub signal_command_kick {
	my ($command, $server, $witem) = @_;

	return unless ($server and defined($server));

	# Syntax for /kick:
	# KICK [<channel>] <nicks> [<reason>]
	# We want to isolate <reason> to know how long it is

	# delete [<channel>] <nicks>
	$command =~ s/^\s*           # Start of String and optional some whitespace
	              (?:            # Grouping
	              \#\S+          # This is <channel>
	              )?             # End of Grouping, <channel> is optional
	              [ ]?           # Maybe a single space
	              \S+            # Everything not whitespace. These are the nicks.
	              [ ]?           # Maybe a single space
	             //x;            # Delete everything

	# The rest of $command is the kickmsg
	my $kickmsg = $command;

	my $len = length($kickmsg);
	my $maxlen = $data{"$server->{'tag'}"}{'kicklen'};

	if ($maxlen > 0 && $len > $maxlen) {
		print CLIENTCRAP "len.pl: kickmsg too long! ($len/$maxlen)";
		Irssi::signal_stop();
	}
}

sub signal_command_part {
	my ($command, $server, $witem) = @_;

	return unless ($server and defined($server));

	# Syntax for /part:
	# PART [<channels>] [<message>]
	# So we want to get rid of the channels to isolate the partmsg

	# Delete [<channels>]
	$command =~ s/^#\S+ //;

	# The rest of $command is the partmsg
	my $partmsg = $command;

	my $len = length($partmsg);
	my $maxlen = $data{"$server->{'tag'}"}{'partlen'};

	if ($maxlen > 0 && $len > $maxlen) {
		print CLIENTCRAP "len.pl: partmsg too long! ($len/$maxlen)";
		Irssi::signal_stop();
	}
}

sub signal_command_quit {
	my ($quitmsg, $server, $witem) = @_;

	return unless ($server and defined($server));

	my $len = length($quitmsg);
	my $maxlen = $data{"$server->{'tag'}"}{'quitlen'};

	if ($maxlen > 0 && $len > $maxlen) {
		print CLIENTCRAP "len.pl: quitmsg too long! ($len/$maxlen)";
		Irssi::signal_stop();
	}
}

# Helper subroutines

sub fix {
	my $string = shift;
	$string =~ s/^\t+//gm;
	return $string;
}

print CLIENTCRAP "len.pl $VERSION loaded. For further information type /len";
