use strict;
use POSIX;
use IO::File;
use vars qw($VERSION %IRSSI);

use Irssi 20020313 qw( settings_add_bool settings_get_bool settings_add_str settings_get_str printformat command_bind);
$VERSION = "1.3.2";
%IRSSI = (
  authors     => 'Marcin Rycki',
  contact     => 'derwan@irssi.pl',
  name        => 'mailcount',
  description	=> 'Adds statusbar item mailcount and displays info about new mails',
  license     => 'GNU GPL v2',
  url         => 'http://irssi.pl/~derwan/',
  changed     => 'Mon Nov 25 12:56:24 CET 2002'
);
use Irssi::TextUI;

Irssi::theme_register([
  'mailcount_notify', 'You have new mail in %C$0%n',
  'mailcount_sender', '%R> %CFrom:%n %_$0%_ ($1 bytes)',
  'mailcount_header', '  %c$0:%n $1',
  'mailcount_part', '  %c$[!-6]0%n $1',
]);

settings_add_str('mailcount', 'mailcount_mailbox', $ENV{'MAIL'});
settings_add_str('mailcount', 'mailcount_show_headers', 'from to cc subject date sender');
settings_add_bool('mailcount', 'mailcount_show_parts', 1);
settings_add_bool('mailcount', 'mailcount_run_in_background', 1);

my ($mailcount_sbitem, $active_tag, $active_pid, $timeout_tag, $refresh_only, %cache);

sub mailcount {
  my ($sbitem, $get_size_only) = @_;
  unless (defined $mailcount_sbitem) {
    $sbitem->{min_size} = $sbitem->{max_size} = 0 if ($get_size_only);
    start_mailcount();
    return;
  }
  start_mailcount();
  $sbitem->default_handler($get_size_only, undef, $mailcount_sbitem, 1);
}

sub start_mailcount {
  ($active_pid or !$active_pid && $active_tag) and return or $active_pid = $active_tag = undef;
  my ($reader, $writer) = (IO::File->new, IO::File->new);
  pipe ($reader, $writer);
  if (settings_get_bool('mailcount_run_in_background')) {
     $active_pid = fork();
     if (defined $active_pid) {
       if ($active_pid) {
          close ($writer);
          Irssi::pidwait_add($active_pid);
          $active_tag = Irssi::input_add(fileno($reader), INPUT_READ, \&input_read, $reader);
          return;
       }
       close ($reader);
       Irssi::timeout_remove($timeout_tag) if $timeout_tag;
       check_mailboxes($writer);
       return;
    }
  }
  $active_tag = Irssi::input_add(fileno($reader), INPUT_READ, \&input_read, $reader);
  check_mailboxes($writer);
}

sub get_mails ($) {
  my ($box, $mh, $ref, $header, @mails) = (shift, IO::File->new());
  return unless (-f $box);
  return unless open($mh, "<$box");
  foreach my $data (<$mh>) {
    if ($data =~ m/^From (.*)\n/ .. $data =~ m/^$/) {
      chomp $data;
      (!$ref or $1 && $ref->{mail} ne $1) and $ref = { mail => $1, size => 0 }, push @mails, $ref;
      ($data =~ m/^([\w_\-]+)\:? (..*)/) and $header = lc($1), $ref->{head}->{$header} = $2 or
          ($data =~ m/\s+(..*)/ && $header) and $ref->{head}->{$header} .= " $1";
      ($data =~ m/\bboundary=\"?([^\"\; ]+)/ && $header eq 'content-type') and $ref->{boundary} = $1;
    } elsif ($ref and $ref->{boundary} && $data eq "--$ref->{boundary}\n" ..
         $data =~ m/^Content\-Type\: ([^\; \n]+)|^$/ and $1) {
        push @{$ref->{parts}}, $1;
    }
    ($ref) and $ref->{size} += length($data) + 1;
  }
  close $mh;
  return @mails;
}

sub check_mailboxes ($) {
  my $headers = settings_get_str('mailcount_show_headers') or 'from';
  my $show_parts = settings_get_bool('mailcount_show_parts') or 0;
  my ($writer, $new, $old) = (shift, 0 , 0);
  foreach my $mbox (split ":", settings_get_str('mailcount_mailbox')) {
    my ($n, $o, $i);
    -f $mbox or next;
    if (my $ctime = (stat($mbox))[9] != $cache{mbox}->{$mbox}->{ctime}) {
      print ($writer "ctime $ctime $mbox\n");
      foreach my $ref (get_mails($mbox)) {
        ($ref->{head}->{subject} =~ m/.*\bfolder internal data/i) and next;
        ($ref->{head}->{status} eq 'RO') and ++$o, next or $n++;
        unless (skip_mail($ref)) {
          print ($writer "notify $mbox\n") unless $i++;
          print ($writer "register $ref->{mail}\nsender $ref->{head}->{from} $ref->{size}\n");
          while ($headers =~ m/([^ ]+)/g) {
            (defined $ref->{head}->{$1} and $1 ne 'from') and print ($writer "header " . ucfirst($1) . " $ref->{head}->{$1}\n");
          }
          unless ($show_parts) {
            for (my $part = 0; $part <= $#{$ref->{parts}}; $part++) {
              print ($writer "part ".($part + 1)." $ref->{parts}[$part]\n");
            }
          }
        }
        purge ($ref);
      }
    } else {
      $n = $cache{mbox}->{$mbox}->{n} or 0;
      $o = $cache{mbox}->{$mbox}->{o} or 0;
    }
    print ($writer "mails $n $o $mbox\n");
    $new+=$n, $old+=$o;
  }
  print ($writer "close $new $old\n");
  close ($writer);
  ($active_pid eq '0') and POSIX::_exit(1);
}

sub skip_mail ($) {
  my $ref = shift();
  my $ret = (grep { $_ eq $ref->{mail} } @{$cache{register}}) ? 1 : 0;
  return $ret;
}

sub purge ($) {
  my $ref = shift;
  (defined $ref->{parts}) and @{$ref->{parts}} = ();
  %{$ref->{head}} = %{$ref} = ();
}

sub input_read ($) {
  my $reader = shift;
  while (<$reader>) {
    chomp;
    /^close (\d+) (\d+)/ and make_sbitem($1, $2), last;
    /^ctime (\d+) (.*)/ and $cache{mbox}->{$2}->{ctime} = $1, next;
    /^mails (\d+) (\d+) (.*)/ and $cache{mbox}->{$3}->{n} = $1, $cache{mbox}->{$3}->{o} = $2, next;
    /^register (.*)/ and push (@{$cache{register}}, $1), next;
    /^notify (.*)/ and printformat(MSGLEVEL_CRAP, 'mailcount_notify', $1), next;
    /^sender (.*) (.*)/ and printformat(MSGLEVEL_CRAP, 'mailcount_sender', $1, $2), next;
    /^header ([^ ]+) (.*)/ and printformat(MSGLEVEL_CRAP, 'mailcount_header', $1, $2), next;
    /^part (\d+) (.*)/ and printformat(MSGLEVEL_CRAP, 'mailcount_part', (($1 == 1) ? 'Parts:' : undef), $2), next;
  }
  Irssi::input_remove($active_tag);
  close $reader;
  $active_tag = $active_pid = undef;
  unless ($timeout_tag) {
    Irssi::statusbar_item_register('mailcount', '{sb Mail: $0-}', 'mailcount');
    $timeout_tag = Irssi::timeout_add(60000, \&refresh, undef);
  }
}

sub refresh {
  Irssi::statusbar_items_redraw('mailcount');
}

sub make_sbitem ($$) {
  my ($new, $old) = @_;
  $mailcount_sbitem = (($new > 0) ? "n/$new" . (($old > 0) ? " o/$old" : undef) : "o/$old");
}

start_mailcount;
