# trackbar.pl
# 
# This little script will do just one thing: it will draw a line each time you
# switch away from a window. This way, you always know just upto where you've
# been reading that window :) It also removes the previous drawn line, so you
# don't see double lines.
#
# Usage: 
#
#     The script works right out of the box, but if you want you can change
#     the working by /set'ing the following variables:
#
#     trackbar_string       The characters to repeat to draw the bar
#     trackbar_style        The style for the bar, %r is red for example
#                           See formats.txt that came with irssi
#
#
# Version history:
#  1.2: - Support for utf-8
#       - How the bar looks can now be configured with trackbar_string 
#         and trackbar_style
#  1.1: - Fixed bug when closing window
#  1.0: - Initial release
#
# Known bugs:
#  - if you /clear a window, it will be uncleared when returning to the window
#    This is probably a bug in irssi
#  - if you resize your irssi (in xterm or so) the bar is not resized 
#    (but will be on leaving the channel)
#  - changing the trackbar style is only visible after returning to a window
#
# Whishlist/todo:
#  - /mark to put the line on the bottom
#  - instead of drawing a line, just invert timestamp or something, 
#    to save a line
#

use strict;
use warnings;
use 5.6.1;
use Irssi;
use Irssi::TextUI;

our $VERSION = "1.2";

our %IRSSI = (
    authors     => q "Peter 'kinlo' Leurs and Johan 'Ion' Kiviniemi",
    contact     => q "peter@pfoe.be",
    name        => q "trackbar",
    description => q "Shows a bar where you've last read a window",
    license     => q "GPLv2",
    url         => q "http://www.pfoe.be/~peter/trackbar/",
    changed     => q "Sun Dec  1 16:38:05 2002",
);

our %config;
$config{'term_utf8'} = lc(Irssi::settings_get_str('term_type')) eq 'utf-8';

# UTF-8 char 9476 is "box drawings light triple dash horizontal".
Irssi::settings_add_str('trackbar',
    'trackbar_string' => $config{'term_utf8'} ? pack('U*', 9476) : '-');
$config{'trackbar_string'} = Irssi::settings_get_str('trackbar_string');

Irssi::settings_add_str('trackbar', 'trackbar_style' => '%K');
$config{'trackbar_style'} = Irssi::settings_get_str('trackbar_style');

Irssi::signal_add(
    'setup changed' => sub {
        $config{'term_utf8'} =
          lc(Irssi::settings_get_str('term_type')) eq 'utf-8';
        $config{'trackbar_string'} = Irssi::settings_get_str('trackbar_string');
        $config{'trackbar_style'}  = Irssi::settings_get_str('trackbar_style');
        if ($config{'trackbar_style'} =~ /(?<!%)[^%]|%%|%$/) {
            Irssi::print(
                "trackbar: %RWarning!%n 'trackbar_style' seems to contain "
                . "printable characters. Only use format codes (read "
                . "formats.txt).", MSGLEVEL_CLIENTERROR);
        }
    }
);

Irssi::signal_add(
    'window changed' => sub {
        my (undef, $oldwindow) = @_;

        if ($oldwindow) {
            my $line = $oldwindow->view()->get_bookmark('trackbar');
            $oldwindow->view()->remove_line($line) if defined $line;
            $oldwindow->print(line($oldwindow->{'width'}), MSGLEVEL_NEVER);
            $oldwindow->view()->set_bookmark_bottom('trackbar');
        }
    }
);

sub line {
    my $width  = shift;
    my $string = $config{'trackbar_string'};
    $string = '-' unless defined $string;

    # This is a horrible kludge. I guess this is Irssi's bug:
    #   my $quake = pack 'U*', 8364;    # EUR symbol
    #   Irssi::settings_add_str 'temp', 'temp_foo' => $quake;
    #   Irssi::print length $quake;
    #       # prints 1
    #   Irssi::print length Irssi::settings_get_str 'temp_foo';
    #       # prints 3
    #
    # According to perlunicode(1), "Beginning with version 5.6, Perl uses
    # logically wide characters to represent strings internally. This internal
    # representation of strings uses the UTF-8 encoding."
    # Maybe Irssi::settings_get_str() should return stuff in that "internal
    # UTF-8 representation"?
    if ($config{'term_utf8'}) {
        $string = pack 'U*', unpack 'U*', $string;
    }

    my $length = length $string;
    if ($length == 0) {
        $string = '-';
        $length = 1;
    }

    my $times = $width / $length;
    $times = int(1 + $times) if $times != int($times);
    $string =~ s/%/%%/g;
    return $config{'trackbar_style'} . substr($string x $times, 0, $width);
}

